#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.zmi.term module

This module provides all terms related management components.
"""
from zope.interface import alsoProvides

from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.view import IModalEditForm
from pyams_skin.schema.button import ActionButton
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_thesaurus.interfaces import MANAGE_THESAURUS_CONTENT_PERMISSION
from pyams_thesaurus.interfaces.term import IThesaurusTerm
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_thesaurus.zmi.thesaurus import thesaurus_modal_form_title
from pyams_thesaurus.zmi.tree import ThesaurusTermsTreeView
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import get_object_factory
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IFormTitle, IModalDisplayFormButtons, IModalEditFormButtons, \
    check_submit_button
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager

__docformat__ = 'restructuredtext'

from pyams_thesaurus import _  # pylint: disable=ungrouped-imports


@viewlet_config(name='add-term.menu',
                context=IThesaurus, layer=IAdminLayer, view=ThesaurusTermsTreeView,
                manager=IToolbarViewletManager, weight=10,
                permission=MANAGE_THESAURUS_CONTENT_PERMISSION)
class ThesaurusTermAddAction(ContextAddAction):
    """Term add menu"""

    label = _("Add term")
    href = 'add-term.html'


@ajax_form_config(name='add-term.html',
                  context=IThesaurus, layer=IPyAMSLayer,
                  permission=MANAGE_THESAURUS_CONTENT_PERMISSION)
class ThesaurusTermAddForm(AdminModalAddForm):
    """Term add form"""

    subtitle = _("New term")
    legend = _("New term properties")

    fields = Fields(IThesaurusTerm).select('label', 'alt', 'definition', 'note', 'generic',
                                           'order', 'associations', 'usage', 'extensions',
                                           'status', 'created')
    content_factory = IThesaurusTerm

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        for name in ('generic', 'associations', 'usage'):
            if name in self.widgets:
                self.widgets[name].thesaurus_name = self.context.name

    def create(self, data):
        factory = get_object_factory(self.content_factory)
        if factory is not None:
            return factory(data.get('label'))
        raise NotImplementedError

    def update_content(self, obj, data):
        changes = super().update_content(obj, data)
        generic = obj.generic
        usage = obj.usage
        if (generic is None) and (usage is None):
            IThesaurus(self.context).top_terms += [obj]
        else:
            if generic is not None:
                generic.specifics += [obj]
            elif usage is not None:
                usage.used_for += [obj]
        return changes

    def add(self, obj):
        self.context.terms[obj.label] = obj


@adapter_config(required=(IThesaurus, IAdminLayer, ThesaurusTermAddForm),
                provides=IAJAXFormRenderer)
class ThesaurusTermAddFormRenderer(ContextRequestViewAdapter):
    """Thesaurus term add form AJAX renderer"""

    def render(self, changes):  # pylint: disable=no-self-use
        """JSON form renderer"""
        if changes is None:
            return None
        term = IThesaurusTerm(changes)
        label = term.label.replace("'", "&#039;")
        if term.generic is None:
            return {
                'status': 'reload'
            }
        return {
            'status': 'success',
            'callbacks': [{
                'callback': 'MyAMS.thesaurus.tree.findTerm',
                'options': {
                    'action': 'add',
                    'term': label
                }
            }]
        }


@adapter_config(required=(IThesaurusTerm, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def thesaurus_term_form_title(context, request, form):
    """Thesaurus term form title getter"""
    thesaurus = get_parent(context, IThesaurus)
    return thesaurus_modal_form_title(thesaurus, request, form)


class IThesaurusTermEditFormButtons(IModalEditFormButtons):
    """Thesaurus term edit form buttons interface"""

    delete = ActionButton(name='delete',
                          title=_("Delete term"),
                          condition=check_submit_button)


@ajax_form_config(name='properties.html',
                  context=IThesaurusTerm, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusTermEditForm(AdminModalEditForm):
    """Thesaurus term edit form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("Term: {}")).format(self.context.label)

    legend = _("Thesaurus term properties")

    @property
    def buttons(self):
        """Buttons getter"""
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        if self.context.is_deletable():
            buttons = ('delete', 'apply', 'close')
        else:
            buttons = ('apply', 'close')
        return Buttons(IThesaurusTermEditFormButtons).select(*buttons)

    fields = Fields(IThesaurusTerm).select('label', 'alt', 'definition', 'note', 'generic',
                                           'order', 'specifics', 'associations', 'usage',
                                           'used_for', 'extracts', 'extensions', 'status',
                                           'created', 'modified')

    generic_changed = False
    usage_changed = False

    def update_actions(self):
        """Actions update"""
        super().update_actions()
        delete_action = self.actions.get('delete')
        if delete_action is not None:
            delete_action.add_class('btn-danger mr-auto')
            delete_action.add_class('submit')
            delete_action.object_data = {
                'ams-click-handler': 'MyAMS.form.submitForm'
            }
            alsoProvides(delete_action, IObjectData)

    def update_widgets(self, prefix=None):
        """Widgets update"""
        super().update_widgets(prefix)
        thesaurus = get_parent(self.context, IThesaurus)
        for name in ('generic', 'specifics', 'associations', 'usage', 'used_for'):
            if name in self.widgets:
                self.widgets[name].thesaurus_name = thesaurus.name
        for name in ('specifics', 'used_for', 'extracts', 'created'):
            if name in self.widgets:
                self.widgets[name].mode = DISPLAY_MODE

    @handler(IThesaurusTermEditFormButtons['delete'])
    def handle_delete(self, action):
        """Delete term"""
        term = IThesaurusTerm(self.context)
        thesaurus = get_parent(term, IThesaurus)
        if thesaurus is not None:
            thesaurus.remove_term(term)
            self.finished_state.update({
                'action': action,
                'changes': term
            })

    @handler(IThesaurusTermEditFormButtons['apply'])
    def handle_apply(self, action):
        """Apply action button handler"""
        super().handle_apply(self, action)

    def apply_changes(self, data):  # pylint: disable=too-many-branches
        """Apply term changes"""
        term = self.context
        thesaurus = get_parent(term, IThesaurus)
        old_label = term.label
        old_generic = term.generic
        old_usage = term.usage
        changes = super().apply_changes(data)
        # Move term if label changed
        if 'label' in changes.get(IThesaurusTerm, ()):
            terms = thesaurus.terms
            del terms[old_label]  # pylint: disable=unsupported-delete-operation
            terms[term.label] = term  # pylint: disable=unsupported-assignment-operation
        # Check modifications
        self.generic_changed = old_generic != term.generic
        self.usage_changed = old_usage != term.usage
        # Check generic change
        if self.generic_changed:
            if old_generic is not None:
                # Add a previous generic?
                # => remove term from list of previous term's specifics
                specifics = old_generic.specifics
                if term in specifics:
                    specifics.remove(term)
                    old_generic.specifics = specifics
            else:
                # No previous generic?
                # => remove term from thesaurus top terms
                top_terms = thesaurus.top_terms
                if term in top_terms:
                    top_terms.remove(term)
                    thesaurus.top_terms = top_terms
            # Check new value
            if term.generic is None:
                # No generic and not a synonym?
                # => add term to top terms
                if (term.usage is None) and (term not in thesaurus.top_terms):
                    thesaurus.top_terms += [term, ]
            else:
                # New generic?
                # => add term to generic specific terms
                if term not in term.generic.specifics:
                    term.generic.specifics += [term, ]
        # Check usage change
        if self.usage_changed:
            if old_usage is not None:
                # Add previous usage term?
                # => update used_for
                used_for = old_usage.used_for
                if term in used_for:
                    used_for.remove(term)
                    old_usage.used_for = used_for
            # Check new term usage
            if term.usage is None:
                # No usage
                # => maybe a top term...
                if (term.generic is None) and (term not in thesaurus.top_terms):
                    thesaurus.top_terms += [term, ]
            else:
                # Term usage?
                # => remove term from top terms
                top_terms = thesaurus.top_terms
                if term in top_terms:
                    top_terms.remove(term)
                    thesaurus.top_terms = top_terms
                # Add term to usage synonyms
                if term not in term.usage.used_for:
                    term.usage.used_for += [term, ]
        return changes


@adapter_config(name='delete',
                required=(IThesaurusTerm, IAdminLayer, ThesaurusTermEditForm),
                provides=IAJAXFormRenderer)
class ThesaurusTermDeleteFormRenderer(ContextRequestViewAdapter):
    """Thesaurus term edit form delete renderer"""

    def render(self, changes):
        """AJAX edit form renderer on delete action"""
        if changes is None:
            return None
        term = self.context
        label = (term.generic or term).label.replace("'", "&#039;")
        translate = self.request.localizer.translate
        return {
            'status': 'success',
            'message': translate(_("Thesaurus term was deleted successfully.")),
            'callback': 'MyAMS.thesaurus.tree.removeTerm',
            'options': {
                'term': label
            }
        }


@adapter_config(name='apply',
                required=(IThesaurusTerm, IAdminLayer, ThesaurusTermEditForm),
                provides=IAJAXFormRenderer)
class ThesaurusTermEditFormRenderer(ContextRequestViewAdapter):
    """Thesaurus term edit form renderer on apply action"""

    def render(self, changes):
        """AJAX edit form renderer"""
        if changes is None:
            return None
        term = self.context
        term_changes = changes.get(IThesaurusTerm, ())
        if self.view.generic_changed or ('order' in term_changes):
            label = term.label.replace("'", "&#039;")
            return {
                'status': 'reload',
                'postReload': 'MyAMS.thesaurus.tree.findMovedTerm',
                'postReloadOptions': {
                    'term': label
                }
            }
        if ('status' in term_changes) or \
                ('label' in term_changes) or \
                ('extensions' in term_changes):
            label = (term.generic or term).label.replace("'", "&#039;")
            return {
                'status': 'callback',
                'callback': 'MyAMS.thesaurus.tree.updateTerm',
                'options': {
                    'term': label
                }
            }
        return None



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.zmi.widget module

This module defines custom widgets which are used to handle thesaurus terms
selection.
"""

import json
import re

from zope.component import queryMultiAdapter
from zope.interface import directlyProvides, implementer_only
from zope.schema.fieldproperty import FieldProperty
from zope.schema.interfaces import ITitledTokenizedTerm
from zope.schema.vocabulary import SimpleTerm

from pyams_form.browser.select import SelectWidget
from pyams_form.converter import SequenceDataConverter
from pyams_form.interfaces import IDataConverter
from pyams_form.interfaces.widget import IFieldWidget
from pyams_form.widget import FieldWidget
from pyams_layer.interfaces import IFormLayer
from pyams_thesaurus.interfaces import REST_TERMS_SEARCH_PATH, REST_TERMS_SEARCH_ROUTE
from pyams_thesaurus.interfaces.term import IThesaurusTerm, STATUS_ARCHIVED
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_thesaurus.schema import IThesaurusTermField, IThesaurusTermsListField
from pyams_thesaurus.zmi.interfaces import IThesaurusTermWidget, IThesaurusTermsListWidget, \
    IThesaurusTermsTreeWidget
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.zodb import volatile_property

__docformat__ = 'restructuredtext'

from pyams_thesaurus import _  # pylint: disable=ungrouped-imports


class SimpleThesaurusTerm(SimpleTerm):
    """Simple thesaurus term"""

    def __init__(self, value, token=None, title=None):  # pylint: disable=super-init-not-called
        self.value = value
        self.token = token or value
        self.title = title
        if title is not None:
            directlyProvides(self, ITitledTokenizedTerm)


def thesaurus_widget_term_factory(term, widget):
    """Thesaurus term factory"""
    if not IThesaurusTerm.providedBy(term):
        thesaurus = widget.thesaurus
        if thesaurus is not None:
            term = thesaurus.terms.get(term)
    if IThesaurusTerm.providedBy(term):
        return SimpleThesaurusTerm(term.label, title=term.label)
    return None


class ThesaurusWidgetMixin:  # pylint: disable=no-member
    """Thesaurus base widget mixin"""

    def term_factory(self, value):
        """Selected term factory"""
        return thesaurus_widget_term_factory(value, self)

    @volatile_property
    def thesaurus(self):
        """Thesaurus utility getter"""
        content = self.form.get_content()
        if IThesaurusTerm.providedBy(content):
            content = get_parent(content, IThesaurus)
        if IThesaurus.providedBy(content):
            thesaurus = content
        else:
            thesaurus = queryMultiAdapter((content, self.request, self.form),
                                          IThesaurus, name=self.field.__name__)
            if thesaurus is None:
                name = self.thesaurus_name
                if name:
                    thesaurus = query_utility(IThesaurus, name=name)
        return thesaurus

    @property
    def thesaurus_name(self):
        """Thesaurus name getter"""
        return self._thesaurus_name

    @thesaurus_name.setter
    def thesaurus_name(self, value):
        """Thesaurus name setter"""
        if value != self._thesaurus_name:
            del self.thesaurus
            self._thesaurus_name = value
            if self.terms is not None:
                self.terms = None
                self.update_terms()

    def is_selected(self, term):
        """Check for term selection"""
        return term.token in self.value

    @property
    def ajax_params(self):
        """AJAX request params getter"""
        return json.dumps({
            'thesaurus_name': self.thesaurus_name,
            'extract_name': self.extract_name
        })


#
# Term widget
#

SYNONYM = re.compile(r'(.*) \[ .* \]')


@adapter_config(required=(IThesaurusTermField, IThesaurusTermWidget),
                provides=IDataConverter)
class ThesaurusTermDataConverter(SequenceDataConverter):
    """Thesaurus term data converter"""

    def to_widget_value(self, value):
        # Widget expects term label or caption
        if isinstance(value, (set, list, tuple)):
            try:
                value = next(iter(value))
            except StopIteration:
                value = None
        if IThesaurusTerm.providedBy(value):
            value = value.label
        return super().to_widget_value(value)

    def to_field_value(self, value):
        # Field expects thesaurus term instance
        if not value:
            return self.field.missing_value
        thesaurus = self.widget.thesaurus
        if thesaurus is None:
            return None
        if isinstance(value, (set, list, tuple)):
            try:
                value = next(iter(value))
            except StopIteration:
                value = None
        if value == self.widget.no_value_token:
            return None
        match = SYNONYM.match(value)
        if match:
            value = match.groups()[0]
        return thesaurus.terms.get(value)


@implementer_only(IThesaurusTermWidget)
class ThesaurusTermWidget(ThesaurusWidgetMixin, SelectWidget):
    """Thesaurus term widget"""

    placeholder = _("No selected term")

    @property
    def ajax_url(self):
        """AJAX URL getter"""
        return self.request.registry.settings.get(f'{REST_TERMS_SEARCH_ROUTE}_route.path',
                                                  REST_TERMS_SEARCH_PATH)

    _thesaurus_name = FieldProperty(IThesaurusTermWidget['thesaurus_name'])
    extract_name = FieldProperty(IThesaurusTermWidget['extract_name'])


@adapter_config(required=(IThesaurusTermField, IFormLayer),
                provides=IFieldWidget)
def ThesaurusTermFieldWidget(field, request):  # pylint: disable=invalid-name
    """Thesaurus term field widget factory"""
    return FieldWidget(field, ThesaurusTermWidget(request))


#
# Terms list widget
#

@adapter_config(required=(IThesaurusTermsListField, IThesaurusTermsListWidget),
                provides=IDataConverter)
class ThesaurusTermsListDataConverter(SequenceDataConverter):
    """Thesaurus terms list data converter"""

    def to_widget_value(self, value):
        # Widget expects a list of thesaurus terms labels or captions
        if value is self.field.missing_value:
            return []
        return [term.label for term in value]

    def to_field_value(self, value):
        # Field expects a list of thesaurus terms
        if not value:
            return self.field.missing_value
        thesaurus = self.widget.thesaurus
        if thesaurus is None:
            return None
        if isinstance(value, str):
            value = value.split('|')
        for idx, val in enumerate(value):
            if val == self.widget.no_value_token:
                continue
            match = SYNONYM.match(val)
            if match:
                value[idx] = match.groups()[0]
        terms = thesaurus.terms
        return [
            terms.get(term)
            for term in value
            if term != self.widget.no_value_token
        ]


@implementer_only(IThesaurusTermsListWidget)
class ThesaurusTermsListWidget(ThesaurusWidgetMixin, SelectWidget):
    """Thesaurus terms list widget"""

    placeholder = _("No selected term")
    multiple = 'multiple'

    @property
    def ajax_url(self):
        """AJAX URL getter"""
        return self.request.registry.settings.get(f'{REST_TERMS_SEARCH_ROUTE}_route.path',
                                                  REST_TERMS_SEARCH_PATH)

    _thesaurus_name = FieldProperty(IThesaurusTermsListWidget['thesaurus_name'])
    extract_name = FieldProperty(IThesaurusTermsListWidget['extract_name'])


@adapter_config(required=(IThesaurusTermsListField, IFormLayer),
                provides=IFieldWidget)
def ThesaurusTermsListFieldWidget(field, request):  # pylint: disable=invalid-name
    """Thesaurus terms list field widget factory"""
    return FieldWidget(field, ThesaurusTermsListWidget(request))


@implementer_only(IThesaurusTermsTreeWidget)
class ThesaurusTermsTreeWidget(ThesaurusTermsListWidget):
    """Thesaurus terms tree widget"""

    @property
    def top_terms(self):
        """Thesaurus top terms getter"""
        thesaurus = self.thesaurus
        if thesaurus is None:
            return ()
        return sorted(thesaurus.get_top_terms(extract=self.extract_name),
                      key=lambda x: (x.order or 999, x.label))

    def get_subterms(self, term):
        """Get sub-terms from given term"""
        for subterm in term.specifics:
            if (not self.extract_name) or (self.extract_name in subterm.extracts):
                if subterm.status != STATUS_ARCHIVED:
                    yield subterm
                    yield from self.get_subterms(subterm)


def ThesaurusTermsTreeFieldWidget(field, request):  # pylint: disable=invalid-name
    """Thesaurus terms tree field widget factory"""
    return FieldWidget(field, ThesaurusTermsTreeWidget(request))



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.api main module

This module provides terms search function.
"""

__docformat__ = 'restructuredtext'

from hypatia.text import ParseError
from pyramid.view import view_config

from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_thesaurus.interfaces.term import STATUS_ARCHIVED
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_utils.list import unique
from pyams_utils.registry import query_utility


@view_config(name='find-terms.json',
             permission=VIEW_SYSTEM_PERMISSION,
             renderer='json', xhr=True)
def find_terms(request):
    """Find terms matching given query"""
    thesaurus_name = request.params.get('thesaurus_name')
    thesaurus = query_utility(IThesaurus, name=thesaurus_name)
    if thesaurus is None:
        return []
    query = request.params.get('term')
    extract_name = request.params.get('extract_name')
    try:
        return [
            {
                'id': term.label,
                'text': term.label
            }
            for term in thesaurus.find_terms(query, extract_name, exact=True, stemmed=True)
            if term.status != STATUS_ARCHIVED
        ]
    except ParseError:
        return []



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.generations.evolve1 module

This module is updating thesaurus extracts internal terms structure.
"""

__docformat__ = 'restructuredtext'

from pyams_thesaurus.interfaces.thesaurus import IThesaurus, IThesaurusExtracts
from pyams_utils.registry import get_local_registry, get_utilities_for, set_local_registry


def evolve(site):
    """Evolve 1: update thesaurus extracts terms"""
    old_registry = get_local_registry()
    try:
        registry = site.getSiteManager()
        set_local_registry(registry)
        # Update thesaurus extracts
        for name, thesaurus in get_utilities_for(IThesaurus):
            extracts = IThesaurusExtracts(thesaurus, None)
            if (extracts is not None) and (len(extracts) > 0):
                print(f"Updating terms for thesaurus « {name} »")
                for term in thesaurus.terms.values():
                    for extract_name in (term.extracts or ()):
                        extract = extracts.get(extract_name)
                        if extract is not None:
                            terms = extract.terms or set()
                            terms.add(term)
                            extract.terms = terms
    finally:
        set_local_registry(old_registry)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.zmi.thesaurus module

This module provides main thesaurus management components.
"""

from pyramid.events import subscriber
from zope.copy import copy
from zope.interface import Interface, Invalid, implementer

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IAddForm, IDataExtractedEvent, IForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION, VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.interfaces import IColumn
from pyams_thesaurus.interfaces.thesaurus import IThesaurus, IThesaurusInfo, IThesaurusManager
from pyams_thesaurus.zmi import ThesaurusListTable
from pyams_thesaurus.zmi.interfaces import IThesaurusTermsTreeMenu
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.intids import IUniqueID
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, AdminModalAddForm
from pyams_zmi.helper.event import get_json_table_row_add_callback, \
    get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager
from pyams_zmi.table import ActionColumn, TableElementEditor
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.breadcrumb import AdminLayerBreadcrumbItem
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_thesaurus import _  # pylint: disable=ungrouped-imports


@adapter_config(required=(IThesaurus, IAdminLayer, IForm),
                provides=IFormTitle)
def thesaurus_form_title(context, request, form):
    """Thesaurus form title getter"""
    return get_object_label(context, request, form)


@adapter_config(required=(IThesaurus, IAdminLayer, IModalPage),
                provides=IFormTitle)
def thesaurus_modal_form_title(context, request, form):
    """Thesaurus form title"""
    manager = get_parent(context, IThesaurusManager)
    return TITLE_SPAN_BREAK.format(
        get_object_label(manager, request, form),
        get_object_label(context, request, form))


class IThesaurusAddForm(IAddForm):
    """Thesaurus add form marker interface"""


@subscriber(IDataExtractedEvent, form_selector=IThesaurusAddForm)
def handle_new_thesaurus_data_extraction(event):
    """Handle new thesaurus data"""
    name = event.data['name'] or ''
    thesaurus = query_utility(IThesaurus, name=name)
    if thesaurus is not None:
        event.form.widgets.errors += (Invalid(_("Another thesaurus is already "
                                                "registered with this name!")),)


@implementer(IThesaurusAddForm)
class ThesaurusAddFormMixin:
    """Thesaurus add form mixin"""

    legend = _("New thesaurus properties")


@adapter_config(required=(IThesaurusManager, IAdminLayer, IThesaurusAddForm),
                provides=IAJAXFormRenderer)
class ThesaurusAddFormRenderer(ContextRequestViewAdapter):
    """Thesaurus add form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        manager = get_parent(self.context, IThesaurusManager)
        return {
            'callbacks': [
                get_json_table_row_add_callback(manager, self.request,
                                                ThesaurusListTable, changes)
            ]
        }


#
# Thesaurus add form
#

@viewlet_config(name='add-thesaurus.menu',
                context=IThesaurusManager, layer=IAdminLayer, view=ThesaurusListTable,
                manager=IToolbarViewletManager, weight=10,
                permission=MANAGE_SYSTEM_PERMISSION)
class ThesaurusAddAction(ContextAddAction):
    """Thesaurus add menu"""

    label = _("Add thesaurus")
    href = 'add-thesaurus.html'


@ajax_form_config(name='add-thesaurus.html',
                  context=IThesaurusManager, layer=IPyAMSLayer,
                  permission=MANAGE_SYSTEM_PERMISSION)
class ThesaurusAddForm(ThesaurusAddFormMixin, AdminModalAddForm):
    """Thesaurus add form"""

    subtitle = _("New thesaurus")

    fields = Fields(IThesaurusInfo).select('name', 'title', 'subject', 'description',
                                           'language', 'creator', 'publisher', 'created')
    content_factory = IThesaurus

    def add(self, obj):
        oid = IUniqueID(obj).oid
        self.context[oid] = obj


#
# Thesaurus clone form
#

@adapter_config(name='clone',
                required=(IThesaurusManager, IAdminLayer, ThesaurusListTable),
                provides=IColumn)
class ThesaurusCloneColumn(ActionColumn):
    """Thesaurus clone column"""

    hint = _("Clone thesaurus")
    icon_class = 'far fa-clone'

    href = 'clone-thesaurus.html'

    permission = MANAGE_SYSTEM_PERMISSION
    weight = 100


@ajax_form_config(name='clone-thesaurus.html',
                  context=IThesaurus, layer=IPyAMSLayer,
                  permission=MANAGE_SYSTEM_PERMISSION)
class ThesaurusCloneForm(ThesaurusAddFormMixin, AdminModalAddForm):
    """Thesaurus clone form"""

    subtitle = _("Create thesaurus clone")

    fields = Fields(IThesaurusInfo).select('name')

    def create(self, data):
        return copy(self.context)

    def add(self, obj):
        oid = IUniqueID(obj).oid
        self.context.__parent__[oid] = obj


#
# Thesaurus editor components
#

@adapter_config(required=(IThesaurus, IAdminLayer, Interface),
                provides=ITableElementEditor)
class ThesaurusElementEditor(TableElementEditor):
    """Thesaurus table element editor"""

    view_name = 'admin'
    modal_target = False


@adapter_config(required=(IThesaurus, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class ThesaurusBreadcrumbItem(AdminLayerBreadcrumbItem):
    """Thesaurus breadcrumb item"""

    @property
    def label(self):
        """Breadcrumb label getter"""
        return self.context.name


@viewlet_config(name='properties.menu',
                context=IThesaurus, layer=IAdminLayer,
                manager=IThesaurusTermsTreeMenu, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusPropertiesMenu(NavigationMenuItem):
    """Thesaurus properties menu"""

    label = _("Properties")
    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=IThesaurus, layer=IPyAMSLayer)
class ThesaurusEditForm(AdminEditForm):
    """Thesaurus edit form"""

    legend = _("Thesaurus properties")

    fields = Fields(IThesaurusInfo).select('name', 'title', 'subject', 'description',
                                           'language', 'creator', 'publisher', 'created')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        name = self.widgets.get('name')
        if name is not None:
            name.mode = DISPLAY_MODE


@adapter_config(required=(IThesaurus, IAdminLayer, ThesaurusEditForm),
                provides=IAJAXFormRenderer)
class ThesaurusEditFormAJAXRenderer(ContextRequestViewAdapter):
    """Thesaurus edit form AJAX renderer"""

    def render(self, changes):
        """AJAX result renderer"""
        if not changes:
            return None
        manager = get_parent(self.context, IThesaurusManager)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(manager, self.request,
                                                    ThesaurusListTable, self.context)
            ]
        }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.loader main module

This module defines common thesaurus loaders components and base classes.
"""

from datetime import datetime
from tempfile import TemporaryFile

from BTrees.OOBTree import OOBTree  # pylint: disable=no-name-in-module,import-error
from zope.componentvocabulary.vocabulary import UtilityVocabulary
from zope.interface import alsoProvides, implementer, noLongerProvides
from zope.schema.fieldproperty import FieldProperty

from pyams_thesaurus.interfaces.loader import IThesaurusExporter, IThesaurusExporterHandler, \
    IThesaurusLoader, IThesaurusLoaderHandler, THESAURUS_EXPORTERS_VOCABULARY, \
    THESAURUS_LOADERS_VOCABULARY
from pyams_thesaurus.interfaces.term import IThesaurusLoaderTerm
from pyams_thesaurus.interfaces.thesaurus import IThesaurusDescription
from pyams_thesaurus.term import ThesaurusTerm
from pyams_thesaurus.thesaurus import Thesaurus, ThesaurusTermsContainer
from pyams_utils.request import query_request
from pyams_utils.unicode import translate_string
from pyams_utils.vocabulary import vocabulary_config


__docformat__ = 'restructuredtext'


#
# Thesaurus loader classes
#

@implementer(IThesaurusDescription)
class ThesaurusLoaderDescription:
    """Thesaurus loader description"""

    title = FieldProperty(IThesaurusDescription['title'])
    subject = FieldProperty(IThesaurusDescription['subject'])
    description = FieldProperty(IThesaurusDescription['description'])
    language = FieldProperty(IThesaurusDescription['language'])
    creator = FieldProperty(IThesaurusDescription['creator'])
    publisher = FieldProperty(IThesaurusDescription['publisher'])
    _created = FieldProperty(IThesaurusDescription['created'])

    @property
    def created(self):
        """Creation date getter"""
        return self._created

    @created.setter
    def created(self, value):
        """Creation date setter"""
        if isinstance(value, str):
            try:
                value = datetime.strptime(value, '%Y-%m-%d').date()
            except ValueError:
                value = datetime.today().date()
        self._created = value


class ThesaurusLoaderTerm:  # pylint: disable=too-many-instance-attributes
    """Thesaurus loader term"""

    def __init__(self, label, alt=None, definition=None, note=None,  # pylint: disable=too-many-arguments
                 generic=None, specifics=None, associations=None,
                 usage=None, used_for=None, created=None, modified=None,
                 weight=0, properties=None):
        self.label = label
        self.alt = alt
        self.definition = definition
        self.note = note
        self.generic = generic
        self.specifics = specifics or []
        self.associations = associations or []
        self.usage = usage
        self.used_for = used_for or []
        self.created = created
        self.modified = modified
        self.weight = int(weight)
        self.properties = properties or {}


@implementer(IThesaurusLoaderHandler)
class BaseThesaurusLoaderHandler:
    """Base thesaurus loader handler"""

    def __init__(self, configuration):
        self.configuration = configuration


class XMLThesaurusLoaderHandler(BaseThesaurusLoaderHandler):
    """Base XML thesaurus loader handler"""


@implementer(IThesaurusLoader)
class BaseThesaurusLoader:
    """Base thesaurus loader"""

    handler = None

    def load(self, data, configuration=None):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
        """Terms loader"""
        handler = self.handler(configuration)  # pylint: disable=not-callable
        if isinstance(data, tuple):
            data = data[1]
        if hasattr(data, 'seek'):
            data.seek(0)
        description, terms = handler.read(data, configuration)
        if configuration and configuration.language and not description.language:
            description.language = configuration.language
        key_store = OOBTree()
        store = ThesaurusTermsContainer()
        # first loop to initialize terms
        for key, term in terms.items():
            new_term = ThesaurusTerm(label=term.label,
                                     alt=term.alt,
                                     definition=term.definition,
                                     note=term.note,
                                     created=term.created,
                                     modified=term.modified)
            alsoProvides(new_term, IThesaurusLoaderTerm)
            key_store[key] = store[term.label] = new_term
            noLongerProvides(new_term, IThesaurusLoaderTerm)
        # second loop to update terms links
        for key, term in terms.items():
            new_term = key_store[key]
            # check generic term
            if term.generic:
                target = key_store.get(term.generic)
                if target is None:
                    target = ThesaurusTerm(label=term.generic)
                    alsoProvides(target, IThesaurusLoaderTerm)
                    key_store[target.label] = store[target.label] = target
                new_term.generic = target
            # check specific terms
            if term.specifics:
                for specific in term.specifics:
                    if key_store.get(specific) is None:
                        target = ThesaurusTerm(label=specific)
                        alsoProvides(target, IThesaurusLoaderTerm)
                        key_store[target.label] = store[target.label] = target
                new_term.specifics = [
                    key_store.get(specific)
                    for specific in term.specifics
                ]
                for subterm in new_term.specifics:
                    subterm.generic = new_term
            # check associations
            if term.associations:
                for association in term.associations:
                    if key_store.get(association) is None:
                        target = ThesaurusTerm(label=association)
                        alsoProvides(target, IThesaurusLoaderTerm)
                        key_store[target.label] = store[target.label] = target
                new_term.associations = [
                    key_store.get(association)
                    for association in term.associations
                ]
            # check usage term
            if term.usage:
                target = key_store.get(term.usage)
                if target is None:
                    target = ThesaurusTerm(label=term.usage)
                    alsoProvides(target, IThesaurusLoaderTerm)
                    key_store[target.label] = store[target.label] = target
                new_term.usage = target
                target.used_for = set(target.used_for) | {new_term}
            # check used-for term
            if term.used_for:
                for used in term.used_for:
                    if key_store.get(used) is None:
                        target = ThesaurusTerm(label=used)
                        alsoProvides(target, IThesaurusLoaderTerm)
                        key_store[target.label] = store[target.label] = target
                new_term.used_for = [
                    key_store.get(used)
                    for used in term.used_for
                ]
                for synonym in new_term.used_for:
                    synonym.usage = new_term
        return Thesaurus(description=description, terms=store)


@vocabulary_config(name=THESAURUS_LOADERS_VOCABULARY)
class ThesaurusLoadersVocabulary(UtilityVocabulary):
    """Thesaurus loaders vocabulary"""

    interface = IThesaurusLoader
    nameOnly = True


#
# Thesaurus exporters classes
#

@implementer(IThesaurusExporterHandler)
class BaseThesaurusExporterHandler:
    """Base thesaurus exporter handler"""

    content_type = None

    def __init__(self, configuration):
        self.configuration = configuration


class XMLThesaurusExporterHandler(BaseThesaurusExporterHandler):
    """Base XML thesaurus exporter handler"""

    content_type = 'text/xml'

    def _write(self, thesaurus, configuration=None):
        """Thesaurus exporter internal write method"""
        raise NotImplementedError

    def write(self, thesaurus, output, configuration=None):
        """Thesaurus exporter write method"""
        doc = self._write(thesaurus, configuration)
        doc.write(output, encoding='utf-8', xml_declaration=True,
                  standalone=True, pretty_print=True)
        return {'Content-Type': 'text/xml; encoding=utf-8'}


@implementer(IThesaurusExporter)
class BaseThesaurusExporter:
    """Base thesaurus exporter"""

    handler = None

    def export(self, thesaurus, configuration=None):
        """Thesaurus export"""
        handler = self.handler(configuration)  # pylint: disable=not-callable
        output = TemporaryFile()
        result = handler.write(thesaurus, output, configuration)
        request = query_request()
        if request is not None:
            filename = translate_string(configuration.filename or (thesaurus.name + '.xml'),
                                        escape_slashes=True, force_lower=False, spaces='-')
            request.response.headers.extend({
                'Content-Type': result.get('Content-Type', 'text/plain'),
                'Content-Disposition': 'attachment; filename="{0}"'.format(filename)
            })
        return output


@vocabulary_config(name=THESAURUS_EXPORTERS_VOCABULARY)
class ThesaurusExportersVocabulary(UtilityVocabulary):
    """Thesaurus exporters vocabulary"""

    interface = IThesaurusExporter
    nameOnly = True



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.zmi main module

This module defines main ZMI components used to manage thesaurus.
"""

from fanstatic import Library, Resource
from pyramid.view import view_config
from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION, VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_table.interfaces import IColumn, IValues
from pyams_thesaurus.interfaces.thesaurus import IThesaurusManager
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IControlPanelMenu, IMenuHeader, IPropertiesMenu, \
    ISiteManagementMenu
from pyams_zmi.table import NameColumn, Table, TableAdminView, TableElementEditor, TrashColumn
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.breadcrumb import AdminLayerBreadcrumbItem
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_thesaurus import _  # pylint: disable=ungrouped-imports


#
# Fanstatic resources
#

library = Library('pyams_thesaurus', 'resources')

thesaurus_css = Resource(library, 'css/thesaurus.css',
                         minified='css/thesaurus.min.css')

thesaurus_js = Resource(library, 'js/thesaurus.js',
                        minified='js/thesaurus.min.js')


#
# Thesaurus manager management components
#

THESAURUS_MANAGER_LABEL = _("Thesaurus container")


@adapter_config(required=(IThesaurusManager, IAdminLayer, IForm),
                provides=IFormTitle)
def thesaurus_manager_form_title(context, request, form):
    """Thesaurus manager form title"""
    return get_object_label(context, request, form)


@viewlet_config(name='thesaurus-manager.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IControlPanelMenu, weight=30,
                permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusManagerMenu(NavigationMenuItem):
    """Thesaurus manager menu"""

    label = _("Thesaurus")
    icon_class = 'fas fa-language'

    def __new__(cls, context, request, view, manager):  # pylint: disable=unused-argument
        manager = query_utility(IThesaurusManager)
        if (manager is None) or not manager.show_home_menu:
            return None
        return NavigationMenuItem.__new__(cls)

    def get_href(self):
        """Menu URL getter"""
        manager = get_utility(IThesaurusManager)
        return absolute_url(manager, self.request, 'admin')


@adapter_config(required=(IThesaurusManager, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class ThesaurusManagerBreadcrumbItem(AdminLayerBreadcrumbItem):
    """Thesaurus manager breadcrumb item"""

    label = THESAURUS_MANAGER_LABEL


@adapter_config(required=(IThesaurusManager, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def thesaurus_manager_label(context, request, view):  # pylint: disable=unused-argument
    """Thesaurus manager label"""
    return request.localizer.translate(THESAURUS_MANAGER_LABEL)


@adapter_config(required=(IThesaurusManager, IAdminLayer, Interface, ISiteManagementMenu),
                provides=IMenuHeader)
def thesaurus_manager_menu_header(context, request, view, manager):  # pylint: disable=unused-argument
    """Thesaurus manager menu header"""
    return THESAURUS_MANAGER_LABEL


@adapter_config(required=(IThesaurusManager, IAdminLayer, Interface),
                provides=ITableElementEditor)
class ThesaurusManagerElementEditor(TableElementEditor):
    """Thesaurus manager element editor"""

    view_name = 'admin#thesaurus-list.html'
    modal_target = False


@viewletmanager_config(name='thesaurus-list.menu',
                       context=IThesaurusManager, layer=IAdminLayer,
                       manager=ISiteManagementMenu, weight=10,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IPropertiesMenu)
class ThesaurusListMenu(NavigationMenuItem):
    """Thesaurus list menu"""

    label = _("Thesaurus list")
    icon_class = 'fas fa-language'
    href = '#thesaurus-list.html'


class ThesaurusListTable(Table):
    """Thesaurus list table"""

    display_if_empty = True


@adapter_config(required=(IThesaurusManager, IAdminLayer, ThesaurusListTable),
                provides=IValues)
class ThesaurusListTableValues(ContextRequestViewAdapter):
    """Thesaurus manager table values adapter"""

    @property
    def values(self):
        """SQLAlchemy manager table values getter"""
        yield from self.context.values()


@adapter_config(name='name',
                required=(IThesaurusManager, IAdminLayer, ThesaurusListTable),
                provides=IColumn)
class ThesaurusManagerNameColumn(NameColumn):
    """Thesaurus manager name column"""


@adapter_config(name='trash',
                required=(IThesaurusManager, IAdminLayer, ThesaurusListTable),
                provides=IColumn)
class ThesaurusManagerTrashColumn(TrashColumn):
    """Thesaurus manager trash column"""

    permission = MANAGE_SYSTEM_PERMISSION


@pagelet_config(name='thesaurus-list.html',
                context=IThesaurusManager, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusListView(TableAdminView):
    """Thesaurus manager list view"""

    title = _("Thesaurus list")
    table_class = ThesaurusListTable
    table_label = _("List of thesaurus")

    @property
    def back_url(self):
        """Form back URL getter"""
        return absolute_url(self.request.root, self.request, 'admin#utilities.html')  # pylint: disable=no-member

    back_url_target = None


@view_config(name='delete-element.json',
             context=IThesaurusManager, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
def delete_thesaurus(request):
    """Delete thesaurus"""
    return delete_container_element(request)


#
# Thesaurus manager properties edit form
#

@viewlet_config(name='properties.menu',
                context=IThesaurusManager, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=10,
                permission=MANAGE_SYSTEM_PERMISSION)
class ThesaurusManagerPropertiesMenu(NavigationMenuItem):
    """Thesaurus manager properties menu"""

    label = _("Properties")
    href = '#properties.html'
    modal_target = False


@ajax_form_config(name='properties.html',
                  context=IThesaurusManager, layer=IPyAMSLayer,
                  permission=MANAGE_SYSTEM_PERMISSION)
class ThesaurusManagerPropertiesEditForm(AdminEditForm):
    """Thesaurus manager properties edit form"""

    legend = _("Thesaurus manager properties")

    fields = Fields(IThesaurusManager).omit('__name__', '__parent__')

    label_css_class = 'control-label col-md-1'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.zmi.extract module

This module provides extracts management components.
"""

from html import unescape

from pyramid.decorator import reify
from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPNotFound
from pyramid.view import view_config
from zope.copy import copy
from zope.interface import Interface, Invalid, implementer

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IAddForm, IDataExtractedEvent, \
    IInnerSubForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IContentSuffixViewletManager
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_table.interfaces import IColumn, IValues
from pyams_template.template import template_config
from pyams_thesaurus.interfaces import ADMIN_THESAURUS_PERMISSION, \
    MANAGE_THESAURUS_EXTRACT_PERMISSION
from pyams_thesaurus.interfaces.thesaurus import IThesaurus, IThesaurusExtract, \
    IThesaurusExtractInfo, IThesaurusExtractRoles, IThesaurusExtracts, IThesaurusManager
from pyams_thesaurus.zmi.thesaurus import thesaurus_modal_form_title
from pyams_thesaurus.zmi.tree import ThesaurusTermsView
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.tree import INode, ITree
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import ViewContentProvider, Viewlet, viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalDisplayForm, AdminModalEditForm
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.helper.event import get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, ICompositeView
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager
from pyams_zmi.table import ActionColumn, JsActionColumn, NameColumn, Table, TableElementEditor, \
    TableView, TrashColumn

__docformat__ = 'restructuredtext'

from pyams_thesaurus import _  # pylint: disable=ungrouped-imports


#
# Thesaurus extracts table
#

class ThesaurusExtractsTable(Table):
    """Thesaurus extracts view"""

    css_classes = {
        'table': 'table table-striped table-hover table-sm datatable extracts'
    }

    @reify
    def data_attributes(self):
        attributes = super().data_attributes
        attributes['table'].update({
            'data-ams-order': '1,asc'
        })
        return attributes

    display_if_empty = True


@adapter_config(required=(IThesaurus, IAdminLayer, ThesaurusExtractsTable),
                provides=IValues)
class ThesaurusExtractsValues(ContextRequestViewAdapter):
    """Thesaurus extracts table values"""

    @property
    def values(self):
        """Extracts table values getter"""
        yield from IThesaurusExtracts(self.context).values()


@adapter_config(name='thesaurus-extracts.view',
                required=(IThesaurus, IAdminLayer, ThesaurusTermsView),
                provides=ICompositeView)
class ThesaurusExtractsView(TableView, ViewContentProvider):
    """Thesaurus extracts view"""

    table_class = ThesaurusExtractsTable
    table_label = _("Thesaurus extracts")

    weight = 10

    @property
    def back_url(self):
        """Form back URL getter"""
        manager = get_utility(IThesaurusManager)
        return absolute_url(manager, self.request, 'admin#thesaurus-list.html')

    back_url_target = None


@adapter_config(name='tree',
                required=(IThesaurus, IAdminLayer, ThesaurusExtractsTable),
                provides=IColumn)
class ThesaurusExtractTreeColumn(ActionColumn):
    """Thesaurus extract tree column"""

    hint = _("Display extract terms")
    icon_class = 'fas fa-stream'

    href = 'terms-view.html'
    weight = 1


@adapter_config(name='name',
                required=(IThesaurus, IAdminLayer, ThesaurusExtractsTable),
                provides=IColumn)
class ThesaurusExtractNameColumn(NameColumn):
    """Thesaurus extract name column"""


@adapter_config(name='color',
                required=(IThesaurus, IAdminLayer, ThesaurusExtractsTable),
                provides=IColumn)
class ThesaurusExtractColorColumn(ActionColumn):
    """Thesaurus extract color column"""

    weight = 40

    def render_cell(self, item):
        return '<i class="fas fa-fw fa-square" style="color: #{};"></i>'.format(item.color)


@adapter_config(name='switch',
                required=(IThesaurus, IAdminLayer, ThesaurusExtractsTable),
                provides=IColumn)
class ThesaurusExtractVisibleSwitchColumn(JsActionColumn):
    """Thesaurus extract visible switch column"""

    icon_class = 'far fa-eye switcher'
    hint = _("Show/hide extract")

    href = 'MyAMS.thesaurus.tree.switchExtract'
    modal_target = False

    weight = 50


@adapter_config(name='trash',
                required=(IThesaurus, IAdminLayer, ThesaurusExtractsTable),
                provides=IColumn)
class ThesaurusExtractTrashColumn(TrashColumn):
    """Thesaurus extract trash column"""

    object_data = {
        'ams-modules': 'container',
        'ams-delete-target': 'delete-extract.json'
    }

    permission = ADMIN_THESAURUS_PERMISSION


@view_config(name='delete-extract.json',
             context=IThesaurus, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=ADMIN_THESAURUS_PERMISSION)
def delete_thesaurus_extract(request):
    """Delete thesaurus extract"""
    return delete_container_element(request, IThesaurusExtracts)


#
# Thesaurus extracts add forms
#

class IThesaurusExtractAddForm(IAddForm):
    """Thesaurus extract add form marker interface"""


@subscriber(IDataExtractedEvent, form_selector=IThesaurusExtractAddForm)
def handle_new_thesaurus_extract_data_extraction(event):
    """Handle new thesaurus extract data"""
    name = event.data['name']
    thesaurus = get_parent(event.form.context, IThesaurus)
    extracts = IThesaurusExtracts(thesaurus)
    if name in extracts:
        event.form.widgets.errors += (Invalid(_("Another extract is already "
                                                "registered with this name!")),)


@adapter_config(required=(IThesaurus, IAdminLayer, IThesaurusExtractAddForm),
                provides=IAJAXFormRenderer)
class ThesaurusExtractAddFormRenderer(ContextRequestViewAdapter):
    """Thesaurus extract add form AJAX renderer"""

    def render(self, changes):  # pylint: disable=no-self-use
        """AJAX form renderer"""
        if not changes:
            return None
        return {
            'status': 'reload'
        }


#
# Extract add form
#

@viewlet_config(name='add-extract.menu',
                context=IThesaurus, layer=IAdminLayer, view=ThesaurusExtractsTable,
                manager=IToolbarViewletManager, weight=10,
                permission=ADMIN_THESAURUS_PERMISSION)
class ThesaurusExtractAddAction(ContextAddAction):
    """Thesaurus extract add action"""

    label = _("Add extract")
    href = 'add-extract.html'


@implementer(IThesaurusExtractAddForm)
class ThesaurusExtractAddFormMixin:
    """Thesaurus extract add form mixin"""

    subtitle = _("New thesaurus extract")


@ajax_form_config(name='add-extract.html',
                  context=IThesaurus, layer=IPyAMSLayer,
                  permission=ADMIN_THESAURUS_PERMISSION)
class ThesaurusExtractAddForm(ThesaurusExtractAddFormMixin, AdminModalAddForm):
    """Thesaurus extract add form"""

    legend = _("New extract properties")

    fields = Fields(IThesaurusExtract).omit('terms')
    content_factory = IThesaurusExtract

    def add(self, obj):
        extracts = IThesaurusExtracts(self.context)
        extracts[obj.name] = obj


#
# Thesaurus extract clone form
#

@adapter_config(name='clone',
                required=(IThesaurus, IAdminLayer, ThesaurusExtractsTable),
                provides=IColumn)
class ThesaurusExtractCloneColumn(ActionColumn):
    """Thesaurus extract clone column"""

    hint = _("Clone thesaurus extract")
    icon_class = 'far fa-clone'

    href = 'clone-extract.html'
    permission = ADMIN_THESAURUS_PERMISSION

    weight = 100


@ajax_form_config(name='clone-extract.html',
                  context=IThesaurusExtract, layer=IPyAMSLayer,
                  permission=ADMIN_THESAURUS_PERMISSION)
class ThesaurusExtractCloneForm(ThesaurusExtractAddFormMixin, AdminModalAddForm):
    """Thesaurus extract clone form"""

    legend = _("New extract properties")

    fields = Fields(IThesaurusExtract).select('name')

    def create(self, data):
        return copy(self.context)

    def add(self, obj):
        extracts = IThesaurusExtracts(self.context.__parent__)
        extracts[obj.name] = obj
        source = IThesaurusExtract(self.context)
        for term in source.terms or ():
            obj.add_term(term)


#
# Thesaurus extract edit form
#

@adapter_config(required=(IThesaurusExtract, IAdminLayer, Interface),
                provides=ITableElementEditor)
class ThesaurusExtractElementEditor(TableElementEditor):
    """Thesaurus extract element editor"""

    def __new__(cls, context, request, view):  # pylint: disable=unused-argument
        if not request.has_permission(MANAGE_THESAURUS_EXTRACT_PERMISSION, context=context):
            return None
        return TableElementEditor.__new__(cls)


@ajax_form_config(name='properties.html',
                  context=IThesaurusExtract, layer=IPyAMSLayer,
                  permission=MANAGE_THESAURUS_EXTRACT_PERMISSION)
class ThesaurusExtractEditForm(AdminModalEditForm):
    """Thesaurus extract edit form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("Thesaurus extract: {}")).format(self.context.name)

    legend = _("Extract properties")

    fields = Fields(IThesaurusExtract).omit('terms')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        name = self.widgets.get('name')
        if name is not None:
            name.mode = DISPLAY_MODE


@adapter_config(required=(IThesaurusExtract, IAdminLayer, IModalPage),
                provides=IFormTitle)
def thesaurus_extract_form_title(context, request, form):
    """Thesaurus extract form title"""
    thesaurus = get_parent(context, IThesaurus)
    return thesaurus_modal_form_title(thesaurus, request, form)


@adapter_config(required=(IThesaurusExtract, IAdminLayer, ThesaurusExtractEditForm),
                provides=IAJAXFormRenderer)
class ThesaurusExtractEditFormAJAXRenderer(ContextRequestViewAdapter):
    """Thesaurus extract edit form AJAX renderer"""

    def render(self, changes):
        """AJAX result renderer"""
        if not changes:
            return None
        if 'color' in changes.get(IThesaurusExtractInfo, ()):
            return {
                'status': 'reload'
            }
        thesaurus = get_parent(self.context, IThesaurus)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(thesaurus, self.request,
                                                    ThesaurusExtractsTable, self.context)
            ]
        }


@adapter_config(name='roles',
                required=(IThesaurusExtract, IAdminLayer, ThesaurusExtractEditForm),
                provides=IInnerSubForm)
class ThesaurusExtractRolesEditForm(Group):
    """Thesaurus extract roles edit form"""

    legend = _("Extract roles")

    fields = Fields(IThesaurusExtractRoles)

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        managers = self.widgets.get('managers')
        if managers is not None:
            if not self.request.has_permission(ADMIN_THESAURUS_PERMISSION, context=self.context):
                managers.mode = DISPLAY_MODE


#
# Thesaurus extract tree view
#

@pagelet_config(name='terms-view.html',
                context=IThesaurusExtract, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusExtractTermsView(AdminModalDisplayForm):
    """Thesaurus extract terms view"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("Thesaurus extract: {}")).format(self.context.name)

    legend = _("Extract terms")


@viewlet_config(name='extract-term',
                context=IThesaurusExtract, layer=IAdminLayer, view=ThesaurusExtractTermsView,
                manager=IContentSuffixViewletManager, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/extract-terms.pt', layer=IAdminLayer)
class ThesaurusExtractTermsTree(Viewlet):
    """Thesaurus extract terms tree view"""

    @property
    def tree(self):
        """Terms tree content getter"""
        thesaurus = get_parent(self.context, IThesaurus)
        extract = self.context.name
        return sorted((
            INode(node)
            for node in ITree(thesaurus).get_root_nodes()
            if extract in (node.extracts or ())
        ), key=lambda x: (x.order or 999, x.label))


@view_config(name='get-nodes.json',
             context=IThesaurusExtract, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION,
             renderer='json', xhr=True)
def get_extract_nodes(request):
    """Get thesaurus extract nodes"""
    label = request.params.get('term')
    if label:
        label = unescape(label)
    thesaurus = get_parent(request.context, IThesaurus)
    term = thesaurus.terms.get(label)
    if term is None:
        raise HTTPNotFound
    result = []
    request.context.get_nodes(term, result)
    return {
        'term': label,
        'nodes': result
    }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.zmi.extension module

This module defines base components for extensions management.
"""

from pyams_thesaurus.interfaces import MANAGE_THESAURUS_CONTENT_PERMISSION
from pyams_thesaurus.interfaces.term import IThesaurusTerm
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_zmi.form import AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'


class ThesaurusTermExtensionEditForm(AdminModalEditForm):
    """Thesaurus term extension properties edit form"""

    _edit_permission = MANAGE_THESAURUS_CONTENT_PERMISSION


@adapter_config(required=(IThesaurusTerm, IAdminLayer, ThesaurusTermExtensionEditForm),
                provides=IFormTitle)
def thesaurus_term_extension_form_title(context, request, form):
    """Thesaurus term extension form title"""
    thesaurus = get_parent(context, IThesaurus)
    return TITLE_SPAN_BREAK.format(
        get_object_label(thesaurus, request, form),
        get_object_label(context, request, form, name='form-title'))



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.interfaces.index module

"""

from zope.interface import Interface
from zope.schema import Bool


__docformat__ = 'restructuredtext'

from pyams_thesaurus import _


class IThesaurusTermIndexBase(Interface):
    """Thesaurus term base index interface"""

    include_parents = Bool(title=_("Include term parents into index values"),
                           default=False,
                           required=False)

    include_synonyms = Bool(title=_("Include term synonyms into index values"),
                            default=False,
                            required=False)


class IThesaurusTermFieldIndex(IThesaurusTermIndexBase):
    """Thesaurus term field index interface"""


class IThesaurusTermsListFieldIndex(IThesaurusTermIndexBase):
    """Thesaurus terms list field index interface"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.extension.html.zmi module

This module defines management components of thesaurus term HTML extension.
"""

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_thesaurus.extension.html import IThesaurusTermHTMLInfo, IThesaurusTermHTMLTarget
from pyams_thesaurus.zmi.extension import ThesaurusTermExtensionEditForm


__docformat__ = 'restructuredtext'

from pyams_thesaurus import _


@ajax_form_config(name='html-description.html',
                  context=IThesaurusTermHTMLTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusTermHTMLDescriptionEditForm(ThesaurusTermExtensionEditForm):
    """Thesaurus term HTML description edit form"""

    subtitle = _("Associated HTML description")
    legend = _("HTML description")
    modal_class = 'modal-xl'

    fields = Fields(IThesaurusTermHTMLInfo)

    def get_content(self):
        return IThesaurusTermHTMLInfo(self.context)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.api.rest module

Thesaurus REST API module.
"""

import sys

from colander import MappingSchema, SchemaNode, SequenceSchema, String, drop
from cornice import Service
from cornice.validators import colander_validator
from hypatia.text import ParseError
from pyramid.httpexceptions import HTTPBadRequest, HTTPOk

from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_security.rest import check_cors_origin, set_cors_headers
from pyams_thesaurus.interfaces import REST_EXTRACTS_GETTER_ROUTE, REST_TERMS_SEARCH_ROUTE
from pyams_thesaurus.interfaces.term import STATUS_ARCHIVED
from pyams_thesaurus.interfaces.thesaurus import IThesaurus, IThesaurusExtracts
from pyams_utils.interfaces.form import NO_VALUE_STRING
from pyams_utils.list import unique
from pyams_utils.registry import query_utility
from pyams_utils.rest import BaseResponseSchema, BaseStatusSchema, STATUS, http_error, rest_responses

__docformat__ = 'restructuredtext'


TEST_MODE = sys.argv[-1].endswith('/test')


#
# Thesaurus extracts getter services
#

class ThesaurusExtractsQuery(MappingSchema):
    """Thesaurus extracts getter schema"""
    thesaurus_name = SchemaNode(String(),
                                description="Selected thesaurus name")


class ThesaurusExtract(MappingSchema):
    """Thesaurus extracts getter result schema"""
    id = SchemaNode(String(),
                    description="Extract ID")
    text = SchemaNode(String(),
                      description="Extract name")


class ThesaurusExtractsList(SequenceSchema):
    """Thesaurus extracts result schema"""
    extract = ThesaurusExtract()


class ThesaurusExtractsSearchResults(BaseStatusSchema):
    """Thesaurus extracts results schema"""
    results = ThesaurusExtractsList(description="Thesaurus extracts list",
                                    missing=drop)


extracts_service = Service(name=REST_EXTRACTS_GETTER_ROUTE,
                           pyramid_route=REST_EXTRACTS_GETTER_ROUTE,
                           description="Thesaurus extracts management")


@extracts_service.options(validators=(check_cors_origin, set_cors_headers))
def extracts_options(request):  # pylint: disable=unused-argument
    """Extracts service OPTIONS handler"""
    return ''


class ThesaurusExtractsRequest(MappingSchema):
    """Thesaurus extracts request"""
    querystring = ThesaurusExtractsQuery()


class ThesaurusExtractsResponse(MappingSchema):
    """Thesaurus extracts getter response"""
    body = ThesaurusExtractsSearchResults()


extracts_get_responses = rest_responses.copy()
extracts_get_responses[HTTPOk.code] = ThesaurusExtractsResponse(
    description="Thesaurus extracts query response")


@extracts_service.get(permission=USE_INTERNAL_API_PERMISSION,
                      schema=ThesaurusExtractsRequest(),
                      validators=(check_cors_origin, colander_validator, set_cors_headers),
                      response_schemas=extracts_get_responses)
def get_extracts(request):
    """Get thesaurus extracts list"""
    params = request.params if TEST_MODE else request.validated.get('querystring', {})
    thesaurus_name = params.get('thesaurus_name')
    if not thesaurus_name:
        return http_error(request, HTTPBadRequest, 'missing argument')
    if thesaurus_name == NO_VALUE_STRING:
        return {
            'status': STATUS.SUCCESS.value,
            'results': []
        }
    thesaurus = query_utility(IThesaurus, name=thesaurus_name)
    if thesaurus is None:
        return http_error(request, HTTPBadRequest, 'bad thesaurus name')
    extracts = IThesaurusExtracts(thesaurus)
    return {
        'status': STATUS.SUCCESS.value,
        'results': [
            {
                'id': extract.name,
                'text': extract.name
            }
            for extract in extracts.values()
        ]
    }


#
# Thesaurus terms search services
#

class ThesaurusTermsQuery(MappingSchema):
    """Thesaurus terms search schema"""
    thesaurus_name = SchemaNode(String(),
                                description="Selected thesaurus name")
    extract_name = SchemaNode(String(),
                              description="Selected extract name",
                              missing=drop)
    term = SchemaNode(String(),
                      description="Terms search string")


class ThesaurusTerm(MappingSchema):
    """Thesaurus term"""
    id = SchemaNode(String(),
                    description="Term ID")
    text = SchemaNode(String(),
                      description="Term label")


class ThesaurusTermsList(SequenceSchema):
    """Thesaurus search results interface"""
    term = ThesaurusTerm()


class ThesaurusTermsSearchResults(BaseResponseSchema):
    """Thesaurus search results schema"""
    results = ThesaurusTermsList(description="Results list",
                                 missing=drop)


terms_service = Service(name=REST_TERMS_SEARCH_ROUTE,
                        pyramid_route=REST_TERMS_SEARCH_ROUTE,
                        description="Thesaurus terms management")


@terms_service.options(validators=(check_cors_origin, set_cors_headers))
def terms_options(request):  # pylint: disable=unused-argument
    """Terms service OPTIONS handler"""
    return ''


class ThesaurusTermsRequest(MappingSchema):
    """Terms getter request"""
    querystring = ThesaurusTermsQuery()


class ThesaurusTermsResponse(MappingSchema):
    """Terms getter response"""
    body = ThesaurusTermsSearchResults()


terms_get_responses = rest_responses.copy()
terms_get_responses[HTTPOk.code] = ThesaurusTermsResponse(
    description="List of terms matching given query")


@terms_service.get(permission=USE_INTERNAL_API_PERMISSION,
                   schema=ThesaurusTermsRequest(),
                   validators=(check_cors_origin, colander_validator, set_cors_headers),
                   response_schemas=terms_get_responses)
def get_terms(request):
    """Returns list of terms matching given query"""
    params = request.params if TEST_MODE else request.validated.get('querystring', {})
    thesaurus_name = params.get('thesaurus_name')
    extract_name = params.get('extract_name')
    query = params.get('term')
    if not (thesaurus_name or query):
        return {
            'status': STATUS.ERROR.value,
            'message': "Missing arguments"
        }
    thesaurus = query_utility(IThesaurus, name=thesaurus_name)
    if thesaurus is None:
        return {
            'status': STATUS.ERROR.value,
            'message': "Thesaurus not found"
        }
    try:
        return {
            'status': STATUS.SUCCESS.value,
            'results': [
                {
                    'id': term.label,
                    'text': term.label
                }
                for term in unique(thesaurus.find_terms(query, extract_name,
                                                        exact=True, stemmed=True))
                if term.status != STATUS_ARCHIVED
            ]
        }
    except ParseError:
        return {}



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_thesaurus.zmi.interfaces module

This module defines thesaurus management components interfaces.
"""

from zope.interface import Interface
from zope.schema import TextLine

from pyams_form.interfaces import INPUT_MODE
from pyams_form.template import widget_template_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.widget import IDynamicSelectWidget


__docformat__ = 'restructuredtext'


class IThesaurusTermsTreeMenu(Interface):
    """Thesaurus terms tree menu marker interface"""


class IThesaurusTermWidget(IDynamicSelectWidget):
    """Single term widget"""

    thesaurus_name = TextLine(title="Thesaurus name",
                              required=False)

    extract_name = TextLine(title="Extract name",
                            required=False)


class IThesaurusTermsListWidget(IDynamicSelectWidget):
    """Terms list widget"""

    thesaurus_name = TextLine(title="Thesaurus name",
                              required=False,
                              default='')

    extract_name = TextLine(title="Extract name",
                            required=False,
                            default='')


@widget_template_config(mode=INPUT_MODE,
                        layer=IPyAMSLayer,
                        template='templates/widget/terms-tree-input.pt')
class IThesaurusTermsTreeWidget(IThesaurusTermsListWidget):
    """Terms tree widget"""
